#include <cstdlib>
#include <fstream>
#include <iostream>

#include "rlcsa.h"


using namespace CSA;


/*
  This program writes BWT of the collection into a file.
*/

int
main(int argc, char** argv)
{
  std::cout << "RLCSA to BWT converter" << std::endl;
  if(argc < 2)
  {
    std::cout << "Usage: read_bwt base_name [buffer_size]" << std::endl;
    return 1;
  }

  std::string base_name = argv[1];
  std::string bwt_name = base_name + ".bwt";
  std::cout << "BWT: " << bwt_name << std::endl;
  std::ofstream bwt_file(bwt_name.c_str(), std::ios_base::binary);
  if(!bwt_file)
  {
    std::cerr << "Error creating BWT file!" << std::endl;
    return 2;
  }
  std::cout << std::endl;

  std::clock_t start = std::clock();
  RLCSA rlcsa(base_name);
  usint buffer_size = 0;
  if(argc > 2) { buffer_size = atoi(argv[2]); }
  usint n = rlcsa.getSize() + rlcsa.getNumberOfSequences();

  if(buffer_size > 0)
  {
    for(usint i = 0; i < n; i += buffer_size)
    {
      pair_type range(i, std::min(i + buffer_size - 1, n - 1));
      uchar* bwt = rlcsa.readBWT(range);
      if(bwt != 0)
      {
        bwt_file.write((char*)bwt, length(range));
        delete[] bwt;
      }
    }
  }
  else
  {
    uchar* bwt = rlcsa.readBWT();
    if(bwt != 0)
    {
      bwt_file.write((char*)bwt, n);
      delete[] bwt;
    }
  }

  std::clock_t stop = std::clock();
  double time = ((stop - start) / (double)CLOCKS_PER_SEC);
//   double bwt_time = ((bwt_stop - bwt_start) / (double)CLOCKS_PER_SEC);
  double megabytes = n / (double)MEGABYTE;
  std::cout << megabytes << " megabytes in " << time << " seconds (" << (megabytes / time) << " MB/s)" << std::endl;
//   std::cout << "BWT created in " << bwt_time << " seconds (" << (megabytes / bwt_time) << " MB/s)" << std::endl;
  std::cout << std::endl;

  bwt_file.close();
  return 0;
}
